﻿using System;
using System.Windows;
using System.Windows.Media.Animation;

namespace Telerik.Windows.QuickStart
{
	public partial class AnimationEventArgs : EventArgs
	{
		public AnimationEventArgs(FrameworkElement element, bool isFinished)
		{
			this.AnimatedObject = element;
			this.IsFinished = isFinished;
		}

		public bool IsFinished { get; set; }
		public FrameworkElement AnimatedObject { get; set; }
	}

	public partial class TimelineAnimationObject
	{
		public TimelineAnimationObject(string propertyPath, Timeline timeline)
		{
			this.PropertyName = propertyPath;
			this.Timeline = timeline;
		}

		public Timeline Timeline { get; set; }
		public string PropertyName { get; set; }
	}

	public partial class CustomStoryboard
	{
		private Storyboard storyBoard;

		public CustomStoryboard(FrameworkElement element)
			: this()
		{
			this.AnimatedObject = element;
		}

		public CustomStoryboard(FrameworkElement element, string propertyName, Timeline timeline)
			: this(element)
		{
			this.AddAnimationTimeline(new TimelineAnimationObject(propertyName, timeline));
		}

		private CustomStoryboard()
		{
			this.storyBoard = new Storyboard();
			this.storyBoard.Completed += this.AnimationOnCompleted;
		}

		public event EventHandler<AnimationEventArgs> AnimationCompleted;

		public FrameworkElement AnimatedObject { get; set; }
		public DoubleAnimation Animation { get; set; }
		public bool IsFinished { get; set; }
		public Duration BeginTime { get; set; }

		public void AddAnimationTimeline(string propertyPath, Timeline timeline)
		{
			this.AddAnimationTimeline(new TimelineAnimationObject(propertyPath, timeline));
		}

		public void AddAnimationTimeline(params TimelineAnimationObject[] timelineAnimationObject)
		{
			foreach (TimelineAnimationObject timelineAnimObj in timelineAnimationObject)
			{
#if !WPF
				Storyboard.SetTarget(timelineAnimObj.Timeline, this.AnimatedObject);
#else
				Storyboard.SetTargetName(timelineAnimObj.Timeline, this.AnimatedObject.Name);
#endif
				Storyboard.SetTargetProperty(timelineAnimObj.Timeline, new PropertyPath(timelineAnimObj.PropertyName));
				this.storyBoard.Children.Add(timelineAnimObj.Timeline);
			}
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "frameworkElement", Justification = "WPF")]
		public void StartAnimation(FrameworkElement frameworkElement)
		{
			this.IsFinished = false;
#if !WPF
			this.storyBoard.Begin();
#else
			this.storyBoard.Begin(frameworkElement, HandoffBehavior.Compose);
#endif
		}

		public void Pause()
		{
#if !WPF
            this.storyBoard.Pause();
#else
			this.storyBoard.Pause(AnimatedObject);
#endif
		}

		public void Stop()
		{
#if !WPF
            this.storyBoard.Pause();
#else
			this.storyBoard.Stop(AnimatedObject);
#endif
		}

		private void AnimationOnCompleted(object sender, System.EventArgs e)
		{
			this.IsFinished = true;
			if (this.AnimationCompleted != null)
			{
				this.AnimationCompleted(this, new AnimationEventArgs(this.AnimatedObject, this.IsFinished));
			}
		}
	}
}
