﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Telerik.Windows.Animation
{
    public class MoveAnimation
    {
        #region [ Fields ]

        private Point? from;
        private Point to;
        private Storyboard story;
        private FrameworkElement animatedObject;
        private double duration = 30;
        private bool finished;
        private RadEasingType type = RadEasingType.Linear;
        private double frame;
        private Point currentPos;
        private Point startPos;
        private TranslateTransform translate;
        #endregion

        #region [ Events ]
        public event EventHandler AnimationCompleted;
        #endregion

        #region [ Properties ]

        public Point? From
        {
            get { return from; }
            set { from = value; }
        }
        public Point To
        {
            get
            {
                return to;
            }
            set
            {
                to = value;
            }
        }
        public FrameworkElement AnimatedObject
        {
            get { return animatedObject; }
            set 
            {
                animatedObject = value;
                if (value != null)
                {
                    GetTranslateTransform();
                }
            }
        }
        public double Duration
        {
            get { return duration; }
            set { duration = value / 20; }
        }
        public bool Finished
        {
            get { return finished; }
        }
        public RadEasingType EasingType
        {
            get { return type; }
            set { type = value; }
        }
        public TimeSpan BeginTime { get; set; }

        #endregion

        #region [ Constructors ]

        public MoveAnimation()
        { }

        public MoveAnimation(FrameworkElement animatedObject)
        {
            this.animatedObject = animatedObject;
            GetTranslateTransform();
            startPos.X = translate.X;
            startPos.Y = translate.Y;
        }

        private void GetTranslateTransform()
        {
            TransformGroup tg = animatedObject.RenderTransform as TransformGroup;
            if (tg == null || !(tg.Children[3] is TranslateTransform))
            {
                CreateDefaultTransformations(animatedObject);
            }
            
            translate = ((animatedObject.RenderTransform as TransformGroup).Children[3] as TranslateTransform);
        }

        public MoveAnimation(FrameworkElement animatedObject, Point to)
            : this(animatedObject)
        {
            this.to = to;
        }

        public MoveAnimation(FrameworkElement animatedObject, Point to, double duration)
            : this(animatedObject)
        {
            this.to = to;
            this.duration = duration;
        }

        public MoveAnimation(FrameworkElement animatedObject, Point from, Point to)
            : this(animatedObject, to)
        {
            this.from = from;
        }

        public MoveAnimation(FrameworkElement animatedObject, Point from, Point to, double duration)
            : this(animatedObject, from, to)
        {
            this.duration = duration;
        }

        #endregion

        #region [ Functions ]
        public void Play()
        {
            finished = false;
            if (animatedObject == null)
            {
                throw new ArgumentException("animatedObject can not be null");
            }
            if (animatedObject.Visibility != Visibility.Visible)
            {
                throw new ArgumentException("animatedObject.Visbility != Visibility.Visible");
            }
            if (story == null)
            {
                story = CreateStoryBoard();
                story.BeginTime = BeginTime;
                story.Duration = TimeSpan.FromMilliseconds(Duration);
                story.Completed += new EventHandler(story_Completed);
            }
            if (!finished)
            {
                story.Stop();
                finished = true;
            }
            if (from.HasValue)
            {
                currentPos.X = translate.X = from.Value.X;
                currentPos.Y = translate.Y = from.Value.Y;
            }
            else
            {
                currentPos.X = translate.X;
                currentPos.Y = translate.Y;
            }

            to.X = to.X - currentPos.X - startPos.X;
            to.Y = to.Y - currentPos.Y - startPos.Y;
            
            frame = 0;
            story.Begin();
        }

        public void Stop()
        {
            if (story != null && story.GetCurrentState() != ClockState.Stopped)
            {
                story.Stop();
                frame = 0;
                finished = true;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1502:AvoidExcessiveComplexity")]
        void story_Completed(object sender, EventArgs e)
        {
            double x = 0;
            double y = 0;

            #region switch statement
            switch (type)
            {
                case RadEasingType.Linear:
                    x = Penner.linear(frame, currentPos.X, to.X, duration);
                    y = Penner.linear(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InQuad:
                    x = Penner.easeInQuad(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInQuad(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutQuad:
                    x = Penner.easeOutQuad(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutQuad(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutQuad:
                    x = Penner.easeInOutQuad(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutQuad(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InCubic:
                    x = Penner.easeInCubic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInCubic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutCubic:
                    x = Penner.easeOutCubic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutCubic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutCubic:
                    x = Penner.easeInOutCubic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutCubic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InQuart:
                    x = Penner.easeInQuart(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInQuart(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutQuart:
                    x = Penner.easeOutQuart(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutQuart(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutQuart:
                    x = Penner.easeInOutQuart(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutQuart(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InQuint:
                    x = Penner.easeInQuint(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInQuint(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutQuint:
                    x = Penner.easeOutQuint(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutQuint(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutQuint:
                    x = Penner.easeInOutQuint(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutQuint(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InSine:
                    x = Penner.easeInSine(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInSine(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutSine:
                    x = Penner.easeOutSine(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutSine(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutSine:
                    x = Penner.easeInOutSine(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutSine(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InExponential:
                    x = Penner.easeInExpo(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInExpo(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutExponential:
                    x = Penner.easeOutExpo(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutExpo(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutExponential:
                    x = Penner.easeInOutExpo(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutExpo(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InCircular:
                    x = Penner.easeInCirc(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInCirc(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutCircular:
                    x = Penner.easeOutCirc(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutCirc(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutCircular:
                    x = Penner.easeInOutCirc(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutCirc(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InElastic:
                    x = Penner.easeInElastic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInElastic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutElastic:
                    x = Penner.easeOutElastic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutElastic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutElastic:
                    x = Penner.easeInOutElastic(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutElastic(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InBack:
                    x = Penner.easeInBack(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInBack(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutBack:
                    x = Penner.easeOutBack(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutBack(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutBack:
                    x = Penner.easeInOutBack(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutBack(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InBounce:
                    x = Penner.easeInBounce(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInBounce(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.OutBounce:
                    x = Penner.easeOutBounce(frame, currentPos.X, to.X, duration);
                    y = Penner.easeOutBounce(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.InOutBounce:
                    x = Penner.easeInOutBounce(frame, currentPos.X, to.X, duration);
                    y = Penner.easeInOutBounce(frame, currentPos.Y, to.Y, duration);
                    break;
                case RadEasingType.Default:
                    x = Penner.linear(frame, currentPos.X, to.X, duration);
                    y = Penner.linear(frame, currentPos.Y, to.Y, duration);
                    break;
                default:
                    break;
            }
            #endregion

            ((animatedObject.RenderTransform as TransformGroup).Children[3] as TranslateTransform).X = x;
            ((animatedObject.RenderTransform as TransformGroup).Children[3] as TranslateTransform).Y = y;

            frame++;
            if (frame <= duration)
            {
                story.BeginTime = TimeSpan.Zero;
                story.Begin();
            }
            else
            {
                finished = true;
                OnAnimationCompleted(EventArgs.Empty);
            }
        }

        protected virtual void OnAnimationCompleted(EventArgs e)
        {
            EventHandler handler = this.AnimationCompleted;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        private static Storyboard CreateStoryBoard()
        {
            //Initialization - all required
            Storyboard story = new Storyboard();
            story.SetValue(FrameworkElement.NameProperty, Guid.NewGuid().ToString());
            //story.SetValue(Storyboard.ChildrenProperty, new DoubleAnimation());
            //story.Children.Clear();

            return story;
        }

        private static void CreateDefaultTransformations(FrameworkElement element)
        {
            TransformGroup group = new TransformGroup();
            ScaleTransform scale = new ScaleTransform();
            SkewTransform skew = new SkewTransform();
            RotateTransform rotate = new RotateTransform();
            TranslateTransform translate = new TranslateTransform();
            group.Children.Add(scale);
            group.Children.Add(skew);
            group.Children.Add(rotate);
            group.Children.Add(translate);
            element.RenderTransform = group;
        }

        #endregion
    }

    public enum RadEasingType
    {
        Linear,
        InQuad,
        OutQuad,
        InOutQuad,
        InCubic,
        OutCubic,
        InOutCubic,
        InQuart,
        OutQuart,
        InOutQuart,
        InQuint,
        OutQuint,
        InOutQuint,
        InSine,
        OutSine,
        InOutSine,
        InExponential,
        OutExponential,
        InOutExponential,
        InCircular,
        OutCircular,
        InOutCircular,
        InElastic,
        OutElastic,
        InOutElastic,
        InBack,
        OutBack,
        InOutBack,
        InBounce,
        OutBounce,
        InOutBounce,
        Default
    }
}
