﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Telerik.Windows.Animation
{
    internal static class Penner
    {

        /*
        Easing Equations (c) 2003 Robert Penner, all rights reserved.
        This work is subject to the terms in http://www.robertpenner.com/easing_terms_of_use.html.
        */

        public static double easeOutElastic(double t, double b, double c, double d)
        {
            if ((t /= d) == 1) return b + c;
            double p = d * .3;
            double s = p / 4;
            return (c * Math.Pow(2, -10 * t) * Math.Sin((t * d - s) * (2 * Math.PI) / p) + c + b);
        }

        public static double easeInElastic(double t, double b, double c, double d)
        {
            if ((t /= d) == 1) return b + c;
            double p = d * .3;
            double s = p / 4;
            return -(c * Math.Pow(2, 10 * (t -= 1)) * Math.Sin((t * d - s) * (2 * Math.PI) / p)) + b;
        }

        public static double easeInOutElastic(double t, double b, double c, double d)
        {
            if ((t /= d / 2) == 2) return b + c;
            double p = d * (.3 * 1.5);
            double s = p / 4;
            if (t < 1) return -.5 * (c * Math.Pow(2, 10 * (t -= 1)) * Math.Sin((t * d - s) * (2 * Math.PI) / p)) + b;
            return c * Math.Pow(2, -10 * (t -= 1)) * Math.Sin((t * d - s) * (2 * Math.PI) / p) * .5 + c + b;
        }

        public static double easeOutBounce(double t, double b, double c, double d)
        {
            if ((t /= d) < (1 / 2.75)) return c * (7.5625 * t * t) + b;
            else if (t < (2 / 2.75)) return c * (7.5625 * (t -= (1.5 / 2.75)) * t + .75) + b;
            else if (t < (2.5 / 2.75)) return c * (7.5625 * (t -= (2.25 / 2.75)) * t + .9375) + b;
            else return c * (7.5625 * (t -= (2.625 / 2.75)) * t + .984375) + b;
        }

        public static double easeInBounce(double t, double b, double c, double d)
        {
            //if ((loadOnDemandTimer /= d) < (1 / 2.75)) return c * (7.5625 * loadOnDemandTimer * loadOnDemandTimer) + b;
            //else if (loadOnDemandTimer < (2 / 2.75)) return c * (7.5625 * (loadOnDemandTimer -= (1.5 / 2.75)) * loadOnDemandTimer + .75) + b;
            //else if (loadOnDemandTimer < (2.5 / 2.75)) return c * (7.5625 * (loadOnDemandTimer -= (2.25 / 2.75)) * loadOnDemandTimer + .9375) + b;
            //else return c * (7.5625 * (loadOnDemandTimer -= (2.625 / 2.75)) * loadOnDemandTimer + .984375) + b;
            return c - easeOutBounce(d - t, 0, c, d) + b;
        }

        public static double easeInOutBounce(double t, double b, double c, double d)
        {
            if (t < d / 2) return easeInBounce(t * 2, 0, c, d) * .5 + b;
            else return easeOutBounce(t * 2 - d, 0, c, d) * .5 + c * .5 + b;
        }

        public static double easeOutExpo(double t, double b, double c, double d)
        {
            return (t == d) ? b + c : c * (-Math.Pow(2, -10 * t / d) + 1) + b;
        }

        public static double easeInExpo(double t, double b, double c, double d)
        {
            return (t == 0) ? b : c * Math.Pow(2, 10 * (t / d - 1)) + b;
        }

        public static double easeInOutExpo(double t, double b, double c, double d)
        {
            if (t == 0) return b;
            if (t == d) return b + c;
            if ((t /= d / 2) < 1) return c / 2 * Math.Pow(2, 10 * (t - 1)) + b;
            return c / 2 * (-Math.Pow(2, -10 * --t) + 2) + b;
        }

        public static double easeOutQuad(double t, double b, double c, double d)
        {
            return -c * (t /= d) * (t - 2) + b;
        }

        public static double easeInQuad(double t, double b, double c, double d)
        {
            return c * (t /= d) * t + b;
        }

        public static double easeInOutQuad(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return c / 2 * t * t + b;
            return -c / 2 * ((--t) * (t - 2) - 1) + b;
        }

        public static double easeOutSine(double t, double b, double c, double d)
        {
            return c * Math.Sin(t / d * (Math.PI / 2)) + b;
        }

        public static double easeInSine(double t, double b, double c, double d)
        {
            return -c * Math.Cos(t / d * (Math.PI / 2)) + c + b;
        }

        public static double easeInOutSine(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return c / 2 * (Math.Sin(Math.PI * t / 2)) + b;
            return -c / 2 * (Math.Cos(Math.PI * --t / 2) - 2) + b;
        }

        public static double easeOutCirc(double t, double b, double c, double d)
        {
            return c * Math.Sqrt(1 - (t = t / d - 1) * t) + b;
        }

        public static double easeInCirc(double t, double b, double c, double d)
        {
            return -c * (Math.Sqrt(1 - (t /= d) * t) - 1) + b;
        }

        public static double easeInOutCirc(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return -c / 2 * (Math.Sqrt(1 - t * t) - 1) + b;
            return c / 2 * (Math.Sqrt(1 - (t -= 2) * t) + 1) + b;
        }

        public static double easeOutCubic(double t, double b, double c, double d)
        {
            return c * ((t = t / d - 1) * t * t + 1) + b;
        }

        public static double easeInCubic(double t, double b, double c, double d)
        {
            return c * (t /= d) * t * t + b;
        }

        public static double easeInOutCubic(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return c / 2 * t * t * t + b;
            return c / 2 * ((t -= 2) * t * t + 2) + b;
        }

        public static double easeOutQuint(double t, double b, double c, double d)
        {
            return c * ((t = t / d - 1) * t * t * t * t + 1) + b;
        }

        public static double easeInQuint(double t, double b, double c, double d)
        {
            return c * (t /= d) * t * t * t * t + b;
        }

        public static double easeInOutQuint(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return c / 2 * t * t * t * t * t + b;
            return c / 2 * ((t -= 2) * t * t * t * t + 2) + b;
        }

        public static double easeOutBack(double t, double b, double c, double d)
        {
            return c * ((t = t / d - 1) * t * ((1.70158 + 1) * t + 1.70158) + 1) + b;
        }

        public static double easeInBack(double t, double b, double c, double d)
        {
            return c * (t /= d) * t * ((1.70158 + 1) * t - 1.70158) + b;
        }

        public static double easeInOutBack(double t, double b, double c, double d)
        {
            double s = 1.70158;
            if ((t /= d / 2) < 1) return c / 2 * (t * t * (((s *= (1.525)) + 1) * t - s)) + b;
            return c / 2 * ((t -= 2) * t * (((s *= (1.525)) + 1) * t + s) + 2) + b;
        }

        public static double easeOutQuart(double t, double b, double c, double d)
        {
            return -c * ((t = t / d - 1) * t * t * t - 1) + b;
        }

        public static double easeInQuart(double t, double b, double c, double d)
        {
            return c * (t /= d) * t * t * t + b;
        }

        public static double easeInOutQuart(double t, double b, double c, double d)
        {
            if ((t /= d / 2) < 1) return c / 2 * t * t * t * t + b;
            return -c / 2 * ((t -= 2) * t * t * t - 2) + b;
        }

        public static double linear(double t, double b, double c, double d)
        {
            return c * t / d + b;
        }
    }
}