﻿#if WPF
using System.Windows.Input;
using System.Windows.Markup;
#endif
using System.Windows.Threading;
#if WPF
using System.Xml;
#endif
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.QuickStart.Parser;
using System.Windows;
using System;
using System.Windows.Documents;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using System.Windows.Media;

namespace Telerik.Windows.QuickStart
{
#if !WPF
	using FlowDocument = TextBlock;
	using FlowDocumentScrollViewer = ScrollViewer;
	using System.Windows.Data;
	using System.Windows.Controls;
	using System.Windows;
	using System.Collections.Generic;
	using System.Text.RegularExpressions;
	using System;
	using System.Windows.Documents;
#endif

	public class BaseCodeViewer : Control
	{
		public static readonly DependencyProperty ExampleItemProperty =
			DependencyProperty.Register("ExampleItem", typeof(ExampleItem), typeof(BaseCodeViewer), null);

		public ExampleItem ExampleItem
		{
			get { return (ExampleItem)GetValue(ExampleItemProperty); }
			set { SetValue(ExampleItemProperty, value); }
		}

		protected FlowDocument FormattedText { get; set; }
		protected FlowDocumentScrollViewer FormattedTextScrollViewer { get; set; }
#if !WPF
		protected TextBox Selection { get; set; }
#endif

		public BaseCodeViewer()
		{
			this.DefaultStyleKey = typeof(BaseCodeViewer);

			this.Loaded += this.OnLoaded;
		}

		private void OnLoaded(object sender, RoutedEventArgs e)
		{
			if (this.ExampleItem != null)
			{
				string relatedFilesToExample = this.ExampleItem.Path + "|" + this.ExampleItem.CommonFilesFolder;
				CodeViewerService.Current.GetSourceFilesListAsync(relatedFilesToExample, this.OnSourceFilesReceived);
			}
		}

		public override void OnApplyTemplate()
		{
			this.InitializeFlowDocument();

			base.OnApplyTemplate();
		}

		private void InitializeFlowDocument()
		{
			this.FormattedTextScrollViewer = this.GetTemplateChild("ScrollViewer") as FlowDocumentScrollViewer;
			this.FormattedText = this.GetTemplateChild("FormattedText") as FlowDocument;
#if !WPF
			this.Selection = (TextBox)this.GetTemplateChild("Selection");
			this.Selection.KeyDown += (sender, e) => { e.Handled = true; };
#endif
		}

		protected virtual void OnSourceFilesReceived(string list)
		{
			Action sourceFilesReceivedAction = () =>
			{
				string code = CodeViewerService.Current.GetDescription(this.ExampleItem.Path);

				InlineCollection inlines = this.ResetCodeViewerContent();

				if (!string.IsNullOrEmpty(code))
				{
					code = ConvertTabsToSpaces(code, 4);

					this.FormatDescription(inlines, code);
				}
				else
				{
					inlines.Add(new Run() { Text = "Loading...." });
				}
			};

			Dispatcher.BeginInvoke(DispatcherPriority.Normal, sourceFilesReceivedAction);
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
		protected void RefreshTextArea()
		{
#if !WPF
			this.Selection.Text = this.FormattedText.Text;
			this.FormattedTextScrollViewer.ScrollToVerticalOffset(0);
			this.FormattedTextScrollViewer.ScrollToHorizontalOffset(0);
#endif
		}

		protected InlineCollection ResetCodeViewerContent()
		{
#if WPF
			Paragraph paragraph = new Paragraph();
			this.FormattedText.Blocks.Clear();
			this.FormattedText.Blocks.Add(paragraph);
#else
			TextBlock paragraph = this.FormattedText;
			paragraph.Inlines.Clear();
#endif
			return paragraph.Inlines;
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "inlineCollection"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "inlines", Justification = "Silverlight compatibility")]
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic", Justification = "WPF compatibility")]
		protected void FormatDescription(InlineCollection inlineCollection, string code)
		{
//#if WPF
//            using (StringReader reader = new StringReader(code))
//            {
//                this.FormattedTextScrollViewer.Document = (FlowDocument)XamlReader.Load(XmlReader.Create(reader));
//                this.FormattedText = this.FormattedTextScrollViewer.Document;

//                FormatCodeFile(inlineCollection, code, ".txt");
//            }
//#else
//            FormatCodeFile(inlineCollection, code, ".txt");
//#endif
			FormatCodeFile(inlineCollection, code, ".txt");

		}

		protected static string ConvertTabsToSpaces(string sourceText, int numberOfSpaces)
		{
			return Regex.Replace(sourceText, "\t", string.Empty.PadLeft(numberOfSpaces, ' '));
		}

		protected static void FormatCodeFile(InlineCollection inlineCollection, string code, string extension)
		{
			List<Token> tokens = new Tokenizer().TokenizeCode(code, extension);
			foreach (Token token in tokens)
			{
				inlineCollection.Add(token.GetRun());
			}
		}
	}
}
