﻿using System;
using System.Globalization;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Resources;

namespace Telerik.Windows.QuickStart
{
	public class SilverlightCodeViewerService : CodeViewerService
	{
		private static string CodeViewerBaseUrl
		{
			get
			{
#if WPF
				return string.Empty;
#else
				// The web service is located at the same folder of the page, containing this application.
				string appUri = System.Windows.Browser.HtmlPage.Document.DocumentUri.AbsoluteUri;

				if (!string.IsNullOrEmpty(System.Windows.Browser.HtmlPage.Document.DocumentUri.Query))
				{
					appUri = appUri.Replace(System.Windows.Browser.HtmlPage.Document.DocumentUri.Query, "");
				}

				int index = appUri.LastIndexOf("Default.aspx", StringComparison.OrdinalIgnoreCase);
				if (index == -1)
				{
					index = appUri.LastIndexOf("Default.VB.aspx", StringComparison.OrdinalIgnoreCase);
				}
				if (index == -1)
				{
					index = appUri.LastIndexOf("#", StringComparison.OrdinalIgnoreCase);
				}
				if (index != -1)
				{
					appUri = appUri.Remove(index, appUri.Length - index);
				}

				string url = appUri + (appUri.EndsWith("/", StringComparison.OrdinalIgnoreCase) ? string.Empty : "/") + "CodeViewer.ashx";

				return url;
#endif
			}
		}

		public override void GetSourceFileAsync(string filePath, Action<string> callback)
		{
			BeginRequest(filePath, false, callback);
		}

		public override void GetSourceFilesListAsync(string examplePath, Action<string> callback)
		{
			BeginRequest(examplePath, true, callback);
		}

		protected override string GetDescriptionPath(string examplePath)
		{
			return examplePath + "/Description.txt";
		}

		protected override Stream GetResourceStream(string path)
		{
			string fullPath = string.Format(CultureInfo.InvariantCulture, "/Examples.{0};component/Examples/{1}", path.Split('/')[0], path);

			StreamResourceInfo sri = Application.GetResourceStream(new Uri(fullPath, UriKind.Relative));
			if (sri == null)
			{
				return null;
			}
			return sri.Stream;
		}

		private static void BeginRequest(string path, bool listFiles, Action<string> callback)
		{
			Uri uri = new Uri(CodeViewerBaseUrl + "?path=" + path + (listFiles ? "&list=true" : string.Empty), UriKind.Absolute);
			WebRequest request = WebRequest.Create(uri);

			AsyncCallback responseCallback = (IAsyncResult result) =>
			{
				WebResponse response = (result.AsyncState as WebRequest).EndGetResponse(result);

				Stream stream = response.GetResponseStream();
				using (TextReader reader = new StreamReader(stream))
				{
					string responseText = reader.ReadToEnd();

					callback(responseText);
				}
			};

			request.BeginGetResponse(responseCallback, request);
		}
	}
}