﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Input;

namespace Telerik.Windows.QuickStart
{
	public static class ScrollViewerExtensions
	{
		public static readonly DependencyProperty EnableMouseWheelProperty =
			DependencyProperty.RegisterAttached("EnableMouseWheel", typeof(bool), typeof(ScrollViewerExtensions), new PropertyMetadata(OnEnableMouseWheelChanged));

		public static bool GetEnableMouseWheel(DependencyObject obj)
		{
			return (bool)obj.GetValue(EnableMouseWheelProperty);
		}

		public static void SetEnableMouseWheel(DependencyObject obj, bool value)
		{
			obj.SetValue(EnableMouseWheelProperty, value);
		}

		private static void OnScrollViewerMouseWheel(object sender, MouseWheelEventArgs args)
		{
			ScrollViewer scrollViewer = sender as ScrollViewer;
			if (scrollViewer != null)
			{
				int scrollDelta = (Math.Abs(args.Delta) / 3) * 40;

				double prevVerticalOffset = scrollViewer.VerticalOffset;
				if (args.Delta > 0)
				{
					scrollViewer.ScrollToVerticalOffset(Math.Max(0, scrollViewer.VerticalOffset - scrollDelta));
				}
				else
				{
					scrollViewer.ScrollToVerticalOffset(Math.Min(scrollViewer.ScrollableHeight, scrollViewer.VerticalOffset + scrollDelta));
				}

				if (prevVerticalOffset != scrollViewer.VerticalOffset)
				{
					args.Handled = true;
				}
			}
		}

		private static void OnEnableMouseWheelChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
		{
			bool enable = (bool)args.NewValue;
			if (enable)
			{
				Mouse.AddMouseWheelHandler(d, new EventHandler<MouseWheelEventArgs>(OnScrollViewerMouseWheel));
			}
			else
			{
				Mouse.RemoveMouseWheelHandler(d, new EventHandler<MouseWheelEventArgs>(OnScrollViewerMouseWheel));
			}
		}
	}
}