﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Markup;
using System.Windows.Media;
using Telerik.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.ComponentModel;
#if WPF
using System.Diagnostics;
using System.Windows.Documents;
using System.Windows.Interop;
using System.Windows.Navigation;
#endif

namespace Telerik.Windows.QuickStart
{
#if WPF
	using RadFrame = System.Windows.Controls.Frame;
#else
	using ComboBox = Telerik.Windows.Controls.RadComboBox;
	using System.Diagnostics;
#endif

	public partial class ExamplePage : BasePage
	{
		#region [ DependencyProperties ]

		public static readonly DependencyProperty ExampleTitleProperty =
			DependencyProperty.Register("ExampleTitle", typeof(string), typeof(ExamplePage), null);

		public static readonly DependencyProperty IsDescriptionViewProperty =
			DependencyProperty.Register("IsDescriptionView", typeof(bool), typeof(ExamplePage), null);

		public static readonly DependencyProperty PreviewControlProperty =
			DependencyProperty.Register("PreviewControl", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty ExampleImageProperty =
			DependencyProperty.Register("ExampleImage", typeof(ImageSource), typeof(ExamplePage), null);

		public static readonly DependencyProperty RightPanelVisibilityProperty =
			DependencyProperty.Register("RightPanelVisibility", typeof(Visibility), typeof(ExamplePage), new PropertyMetadata(Visibility.Visible));

		public static readonly DependencyProperty TopRightPanelProperty =
			DependencyProperty.Register("TopRightPanel", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty BottomRightPanelProperty =
			DependencyProperty.Register("BottomRightPanel", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty OutputProperty =
			DependencyProperty.Register("Output", typeof(string), typeof(ExamplePage), null);

		public static readonly DependencyProperty DropDownProperty =
			DependencyProperty.Register("DropDown", typeof(ComboBox), typeof(ExamplePage), null);

		public string Output
		{
			get { return (string)GetValue(OutputProperty); }
			set { SetValue(OutputProperty, value); }
		}

		public ComboBox DropDown
		{
			get { return (ComboBox)GetValue(DropDownProperty); }
			set { SetValue(DropDownProperty, value); }
		}

		public Panel TopRightPanel
		{
			get { return (Panel)GetValue(TopRightPanelProperty); }
			set { SetValue(TopRightPanelProperty, value); }
		}

		public Panel BottomRightPanel
		{
			get { return (Panel)GetValue(BottomRightPanelProperty); }
			set { SetValue(BottomRightPanelProperty, value); }
		}

		public bool IsDescriptionView
		{
			get { return (bool)GetValue(IsDescriptionViewProperty); }
			set { SetValue(IsDescriptionViewProperty, value); }
		}

		public ImageSource ExampleImage
		{
			get { return (ImageSource)GetValue(ExampleImageProperty); }
			set { SetValue(ExampleImageProperty, value); }
		}

		public Panel PreviewControl
		{
			get { return (Panel)GetValue(PreviewControlProperty); }
			set { SetValue(PreviewControlProperty, value); }
		}

		public Visibility RightPanelVisibility
		{
			get { return (Visibility)GetValue(RightPanelVisibilityProperty); }
			set { SetValue(RightPanelVisibilityProperty, value); }
		}

		public string ExampleTitle
		{
			get { return (string)GetValue(ExampleTitleProperty); }
			set { SetValue(ExampleTitleProperty, value); }
		}

		#endregion

		#region [ User control parts ]
		private ContentControl RightPanel
		{
			get
			{
				var panel = this.FindName("rightPanel") as ContentControl;
				return panel ?? new ContentControl();
			}
		}

		private ContentControl ExampleArea
		{
			get
			{
				var panel = this.FindName("exampleArea") as ContentControl;
				return panel ?? new ContentControl();
			}
		}

		private ContentControl BottomPanel
		{
			get
			{
				var panel = this.FindName("bottomPanel") as ContentControl;
				return panel ?? new ContentControl();
			}
		}

		private Border ExamplePanel
		{
			get
			{
				var panel = this.FindName("exampleblock") as Border;
				return panel ?? new Border();
			}
		}

		private CodeViewer CodeViewer
		{
			get
			{
				var control = this.FindName("codeViewer") as CodeViewer;
				return control ?? new CodeViewer();
			}
		}

		private ComboBox ThemesComboBox
		{
			get
			{
				var control = this.FindName("themesComboBox") as ComboBox;
				return control ?? new ComboBox();
			}
		}
		#endregion

		protected override void OnCurrentExampleChanged(object oldValue, object newValue)
		{
			Example example = newValue as Example;
			if (null == example) return;

			Panel topRightPanel = example.TopRightPanel;
			Panel bPanel = example.BottomRightPanel;

			this.ExampleArea.Content = example;

			if (null != topRightPanel)
			{
				this.RightPanel.Content = topRightPanel;
			}
			if (bPanel != null)
			{
				BottomPanel.Content = bPanel;
			}
			this.ThemesComboBox.Visibility = example.ThemeComboBoxVisibility;

			this.ExamplePanel.Visibility = (bPanel == null && topRightPanel == null) ? Visibility.Collapsed : Visibility.Visible;
		}

		protected override void OnExampleItemChanged(object oldItem, object newItem)
		{
			this.NavigationIdentifier = ((ExampleItem)newItem).Path;
			this.DataContext = newItem;
		}

		public ExamplePage()
		{
			string version = "SL";
#if WPF
			version = "WPF";
#endif
			string resourceLocation = string.Format("Telerik.Windows.QuickStart;component/themes/ExamplePage_{0}.xaml", version);

			Application.LoadComponent(this, new Uri(resourceLocation, UriKind.Relative));
		}

		private static MainPage GetMainPage(DependencyObject example)
		{
			DependencyObject parent = VisualTreeHelper.GetParent(example);
			while (parent != null)
			{
				MainPage page = parent as MainPage;
				if (page != null)
				{
					return page;
				}
				parent = VisualTreeHelper.GetParent(parent);
			}
			return null;
		}

		public void ThemeChanged(object sender, PropertyChangedEventArgs args)
		{
			MainPage mainPage = GetMainPage(this);
			mainPage.Navigate(this.ExampleItem);
		}

		private void OnPreviewButtonClick(object sender, RoutedEventArgs e)
		{
			IsDescriptionView = !IsDescriptionView;
			string stateManagerMode = IsDescriptionView ? "hideExamplePanel" : "showExamplePanel";
			
			if (IsDescriptionView)
			{
				this.CodeViewer.TabControl.SelectedIndex = 0;
			}
			this.SelectedExample.OnSwitchCodeArea(IsDescriptionView);

			Storyboard storyBoard = this.Resources[stateManagerMode] as Storyboard;
			if (null != storyBoard)
			{
				storyBoard.Begin();
			}
		}
	}
}