﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Telerik.Windows.Controls;
using System;
using System.Collections.Generic;

namespace Telerik.Windows.QuickStart
{
	public partial class ExamplePage : BasePage
	{
		#region [ DependancyProperties ]

		public static readonly DependencyProperty ExampleTitleProperty =
			DependencyProperty.Register("ExampleTitle", typeof(string), typeof(ExamplePage), null);

		public static readonly DependencyProperty IsDescriptionViewProperty =
			DependencyProperty.Register("IsDescriptionView", typeof(bool), typeof(ExamplePage), null);

		public static readonly DependencyProperty PreviewControlProperty =
			DependencyProperty.Register("PreviewControl", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty ExampleImageProperty =
			DependencyProperty.Register("ExampleImage", typeof(ImageSource), typeof(ExamplePage), null);

		public static readonly DependencyProperty RightPanelVisibilityProperty =
			DependencyProperty.Register("RightPanelVisibility", typeof(Visibility), typeof(ExamplePage), new PropertyMetadata(Visibility.Visible));

		public static readonly DependencyProperty TopRightPanelProperty =
			DependencyProperty.Register("TopRightPanel", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty BottomRightPanelProperty =
			DependencyProperty.Register("BottomRightPanel", typeof(Panel), typeof(ExamplePage), null);

		public static readonly DependencyProperty OutputProperty =
			DependencyProperty.Register("Output", typeof(string), typeof(ExamplePage), null);

		public static readonly DependencyProperty DropDownProperty =
			DependencyProperty.Register("DropDown", typeof(ComboBox), typeof(ExamplePage), null);

		public string Output
		{
			get { return (string)GetValue(OutputProperty); }
			set { SetValue(OutputProperty, value); }
		}

		public ComboBox DropDown
		{
			get { return (ComboBox)GetValue(DropDownProperty); }
			set { SetValue(DropDownProperty, value); }
		}

		public Panel TopRightPanel
		{
			get { return (Panel)GetValue(TopRightPanelProperty); }
			set { SetValue(TopRightPanelProperty, value); }
		}

		public Panel BottomRightPanel
		{
			get { return (Panel)GetValue(BottomRightPanelProperty); }
			set { SetValue(BottomRightPanelProperty, value); }
		}

		public bool IsDescriptionView
		{
			get { return (bool)GetValue(IsDescriptionViewProperty); }
			set { SetValue(IsDescriptionViewProperty, value); }
		}

		public ImageSource ExampleImage
		{
			get { return (ImageSource)GetValue(ExampleImageProperty); }
			set { SetValue(ExampleImageProperty, value); }
		}

		public Panel PreviewControl
		{
			get { return (Panel)GetValue(PreviewControlProperty); }
			set { SetValue(PreviewControlProperty, value); }
		}

		public Visibility RightPanelVisibility
		{
			get { return (Visibility)GetValue(RightPanelVisibilityProperty); }
			set { SetValue(RightPanelVisibilityProperty, value); }
		}

		public string ExampleTitle
		{
			get { return (string)GetValue(ExampleTitleProperty); }
			set { SetValue(ExampleTitleProperty, value); }
		}
		#endregion

		protected override void OnCurrentExampleChanged(object oldValue, object newValue)
		{
			Example example = newValue as Example;
			if (null == example) return;

			Panel topRightPanel = example.TopRightPanel;
			Panel bPanel = example.BottomRightPanel;

			if (null != exampleArea)
			{
				exampleArea.Content = example;
			}
			if (null != rightPanel && null != topRightPanel)
			{
				rightPanel.Content = topRightPanel;
			}
			if (null != bottomPanel && bPanel != null)
			{
				bottomPanel.Content = bPanel;
			}
			this.themesComboBox.Visibility = example.ThemeComboBoxVisibility;

			exampleblock.Visibility = (bPanel == null && topRightPanel == null) ? Visibility.Collapsed : Visibility.Visible;
		}

		protected override void OnExampleItemChanged(object oldItem, object newItem)
		{
			this.NavigationIdentifier = ((ExampleItem)newItem).Path;

			this.DataContext = newItem;
		}

		public ExamplePage()
		{
			InitializeComponent();

			this.themesComboBox.SelectionChanged += this.OnRadComboBoxItemChanged;

			var themeCollection = new List<string>();
			themeCollection.Add("Themes");
			themeCollection.AddRange(ThemeManager.StandardThemeNames);

			this.themesComboBox.ItemsSource = themeCollection;
			this.themesComboBox.SelectedIndex = StyleManager.ApplicationTheme == null ? 0 :
				ThemeManager.StandardThemeNames.IndexOf(StyleManager.ApplicationTheme.ToString()) + 1;
		}

		private static MainPage GetMainPage(DependencyObject example)
		{
			DependencyObject parent = VisualTreeHelper.GetParent(example);
			while (parent != null)
			{
				MainPage page = parent as MainPage;
				if (page != null)
				{
					return page;
				}
				parent = VisualTreeHelper.GetParent(parent);
			}
			return null;
		}

		private void OnPreviewButtonClick(object sender, RoutedEventArgs e)
		{
			IsDescriptionView = !IsDescriptionView;
			string stateManagerMode = IsDescriptionView ? "hideExamplePanel" : "showExamplePanel";

			if (IsDescriptionView)
			{
				this.codeViewer.TabControl.SelectedIndex = 0;
			}
			this.SelectedExample.OnSwitchCodeArea(IsDescriptionView);
#if !WPF
                Storyboard sb = this.Resources[stateManagerMode] as Storyboard;
                if (null != sb)
                {
                    sb.Begin();
                }
#endif
		}

		private void OnRadComboBoxItemChanged(object sender, SelectionChangedEventArgs e)
		{
			if (this.themesComboBox.SelectedIndex < 1) return;

			string selectedTheme = (string)this.themesComboBox.SelectedItem;

			if (StyleManager.ApplicationTheme != null)
			{
				//Prevent loading the same theme
				if (string.Equals(StyleManager.ApplicationTheme.ToString(), selectedTheme, StringComparison.OrdinalIgnoreCase))
				{
					return;
				}
			}
			StyleManager.ApplicationTheme = ThemeManager.FromName(selectedTheme);

			MainPage mainPage = GetMainPage(this);
			if (mainPage != null)
			{
				mainPage.Navigate(this.ExampleItem);
			}
		}
	}
}