﻿using System.Windows.Input;
using System;
using System.Windows.Controls.Primitives;
using System.Windows;

namespace Telerik.Windows.QuickStart
{
    internal class DragValidator
    {
        // Fields
        private bool draggingActive;
        private Point start;
        private UIElement _targetElement;

        // Events
        public event EventHandler<DragCompletedEventArgs> DragCompletedEvent;

        public event EventHandler<DragDeltaEventArgs> DragDeltaEvent;

        public event EventHandler<DragStartedEventArgs> DragStartedEvent;

        // Methods
        public DragValidator(UIElement targetElement)
        {
            this._targetElement = targetElement;
            this._targetElement.MouseLeftButtonDown += new MouseButtonEventHandler(this.TargetElement_MouseLeftButtonDown);
            this._targetElement.MouseLeftButtonUp += new MouseButtonEventHandler(this.TargetElement_MouseLeftButtonUp);
            this._targetElement.MouseMove += new MouseEventHandler(this.TargetElement_MouseMove);
        }

        private void OnDragCompleted(MouseEventArgs e, bool canceled)
        {
            EventHandler<DragCompletedEventArgs> dragCompletedEvent = this.DragCompletedEvent;
            if (dragCompletedEvent != null)
            {
                Point point;
                if (canceled)
                {
                    point = new Point();
                }
                else
                {
                    Point position = e.GetPosition(this._targetElement);
                    if (this._targetElement.RenderTransform != null)
                    {
                        position = this._targetElement.RenderTransform.Transform(position);
                    }
                    point = new Point(position.X - this.start.X, position.Y - this.start.Y);
                }
                dragCompletedEvent(this._targetElement, new DragCompletedEventArgs(point.X, point.Y, canceled));
            }
        }

        private void OnDragDelta(MouseEventArgs e)
        {
            EventHandler<DragDeltaEventArgs> dragDeltaEvent = this.DragDeltaEvent;
            if (dragDeltaEvent != null)
            {
                Point position = e.GetPosition(this._targetElement);
                if (this._targetElement.RenderTransform != null)
                {
                    position = this._targetElement.RenderTransform.Transform(position);
                }
                dragDeltaEvent(this._targetElement, new DragDeltaEventArgs(position.X - this.start.X, position.Y - this.start.Y));
            }
        }

        private void OnDragStarted()
        {
            EventHandler<DragStartedEventArgs> dragStartedEvent = this.DragStartedEvent;
            if (dragStartedEvent != null)
            {
                dragStartedEvent(this._targetElement, new DragStartedEventArgs(0.0, 0.0));
            }
        }

        private void TargetElement_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.start = e.GetPosition(this._targetElement);
            if (this._targetElement.RenderTransform != null)
            {
                this.start = this._targetElement.RenderTransform.Transform(this.start);
            }
            this.draggingActive = this._targetElement.CaptureMouse();
            if (this.draggingActive)
            {
                this.OnDragStarted();
            }
        }

        private void TargetElement_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            this._targetElement.ReleaseMouseCapture();
            this.draggingActive = false;
            this.OnDragCompleted(e, false);
        }

        private void TargetElement_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.draggingActive)
            {
                this.OnDragDelta(e);
            }
        }
    }
}