﻿using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
    [TemplatePart(Name = "HorizontalTemplate", Type = typeof(FrameworkElement)),
     TemplatePart(Name = "VerticalTemplate", Type = typeof(FrameworkElement))]
    internal class PreviewControl : Control
    {
        // Fields
        private GridSplitter.GridResizeDirection _currentGridResizeDirection;
        internal FrameworkElement _elementHorizontalTemplateFrameworkElement;
        internal FrameworkElement _elementVerticalTemplateFrameworkElement;
        private Point _gridSplitterOrigin = new Point();
        internal const string ElementHorizontalTemplateName = "HorizontalTemplate";
        internal const string ElementVerticalTemplateName = "VerticalTemplate";

        // Methods
        public void Bind(GridSplitter gridSplitter)
        {
            base.Style = gridSplitter.PreviewStyle;
            base.Height = gridSplitter.ActualHeight;
            base.Width = gridSplitter.ActualWidth;
            if (gridSplitter._resizeData != null)
            {
                this._currentGridResizeDirection = gridSplitter._resizeData.ResizeDirection;
            }
            Matrix matrix = ((MatrixTransform)gridSplitter.TransformToVisual((UIElement)gridSplitter.Parent)).Matrix;
            this._gridSplitterOrigin.X = matrix.OffsetX;
            this._gridSplitterOrigin.Y = matrix.OffsetY;
            base.SetValue(Canvas.LeftProperty, this._gridSplitterOrigin.X);
            base.SetValue(Canvas.TopProperty, this._gridSplitterOrigin.Y);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this._elementHorizontalTemplateFrameworkElement = base.GetTemplateChild("HorizontalTemplate") as FrameworkElement;
            this._elementVerticalTemplateFrameworkElement = base.GetTemplateChild("VerticalTemplate") as FrameworkElement;
            if (this._currentGridResizeDirection == GridSplitter.GridResizeDirection.Columns)
            {
                if (this._elementHorizontalTemplateFrameworkElement != null)
                {
                    this._elementHorizontalTemplateFrameworkElement.Visibility = Visibility.Collapsed;
                }
                if (this._elementVerticalTemplateFrameworkElement != null)
                {
                    this._elementVerticalTemplateFrameworkElement.Visibility = Visibility.Visible;
                }
            }
            else
            {
                if (this._elementHorizontalTemplateFrameworkElement != null)
                {
                    this._elementHorizontalTemplateFrameworkElement.Visibility = Visibility.Visible;
                }
                if (this._elementVerticalTemplateFrameworkElement != null)
                {
                    this._elementVerticalTemplateFrameworkElement.Visibility = Visibility.Collapsed;
                }
            }
        }

        // Properties
        public double OffsetX
        {
            get
            {
                return (((double)base.GetValue(Canvas.LeftProperty)) - this._gridSplitterOrigin.X);
            }
            set
            {
                base.SetValue(Canvas.LeftProperty, this._gridSplitterOrigin.X + value);
            }
        }

        public double OffsetY
        {
            get
            {
                return (((double)base.GetValue(Canvas.TopProperty)) - this._gridSplitterOrigin.Y);
            }
            set
            {
                base.SetValue(Canvas.TopProperty, this._gridSplitterOrigin.Y + value);
            }
        }
    }
} 
