﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Resources;
using System.Text;
using System.Windows;
using System.Windows.Resources;
using System.Xml.Linq;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
#if WPF
	using System.Globalization;
	using System.Windows.Controls;
	using System.Windows.Input;
	using Telerik.Windows.Data;
	using RadFrame = System.Windows.Controls.Frame;
#else
	using System.Globalization;
	using System.Text.RegularExpressions;
	using System.Windows.Controls;
	using System.Windows.Input;
	using Telerik.Windows.QuickStart.Animation;
	using Page = System.Windows.Controls.Control;
	using SelectionChangedEventArgs = Telerik.Windows.Controls.SelectionChangedEventArgs;
#endif

	public class MainPage : Page
	{
		private RadTreeView examplesTree;
        private ExampleItem CurrentSelectedExample;
		private RadFrame exampleContainer;
		private WatermarkedTextBox searchTextBox;
#if !WPF
        private LoadingSpin LoadingVisual;
		private FrameworkElement splashLoader;
		private FrameworkElement topLeftLogo;
		private FrameworkElement layoutRoot;
		private FrameworkElement topRightButtons;
		private FrameworkElement topRightPanel;
		private FrameworkElement searchBox;
		private FrameworkElement treeViewPanel;
		private FrameworkElement examplesPanel;
		private FrameworkElement examplesMask;
#endif

		public bool AsyncDownloadExamples { get; set; }

		public MainPage()
		{
			this.DefaultStyleKey = typeof(MainPage);
		}

		private ExamplesCollection Examples { get; set; }

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			this.exampleContainer = (RadFrame)this.GetTemplateChild("ExampleContainer");
#if !WPF
			this.LoadingVisual = (LoadingSpin)this.GetTemplateChild("LoadingVisual");
#endif

//            Button closeAwarePanel = this.GetTemplateChild("closeAwardPanel") as Button;
//            Grid awardPanel = this.GetTemplateChild("awardPanel") as Grid;

//            if (null != closeAwarePanel && null != awardPanel)
//            {
//                DateTime endDate = new DateTime(2009, 04, 26);

//                if (endDate < DateTime.Now.Date)
//                {
//                    awardPanel.Visibility = Visibility.Collapsed;
//                }
//                closeAwarePanel.Click += (sender, e) =>
//                {
//#if !WPF
//                    VisualStateManager.GoToState(this, "HideAwardPanel", false);
//#endif
//                };
//            }

			Button searchButton = (Button)this.GetTemplateChild("searchButton");
			if (searchButton != null)
			{
				searchButton.Click += (sender, e) => { this.SearchForExample(); };
			}

			if (this.searchTextBox != null)
			{
				this.searchTextBox.WatermarkedTextChanged -= this.OnSearchTextChanged;
				this.searchTextBox.KeyDown -= this.OnSearchTextKeyDown;
			}
			this.searchTextBox = (WatermarkedTextBox)this.GetTemplateChild("searchTextBox");
			if (this.searchTextBox != null)
			{
				this.searchTextBox.WatermarkedTextChanged += this.OnSearchTextChanged;
				this.searchTextBox.KeyDown += this.OnSearchTextKeyDown;
			}
#if !WPF
			NavigationService service = NavigationService.GetNavigationService();
			service.RootPanel = (RadFrameContainer)this.GetTemplateChild("radFameContainer");

			HyperlinkButton fullscreenButton = this.GetTemplateChild("fullscreenButton") as HyperlinkButton;
			fullscreenButton.Click += (sender, e) =>
			{
				Application.Current.Host.Content.IsFullScreen = !Application.Current.Host.Content.IsFullScreen;
			};

			BrowserHistoryManager.BrowserHistoryNavigate += (sender, e) => { OnBrowserHistoryNavigateCompleted(e.Path); };
#endif
			if (this.examplesTree != null)
			{
				this.examplesTree.Selected -= this.OnExamplesTreeSelectionChanged;
			}
			this.examplesTree = (RadTreeView)this.GetTemplateChild("ExamplesTree");
			this.examplesTree.Selected += this.OnExamplesTreeSelectionChanged;

			//Start animations:
#if !WPF
			splashLoader = this.GetTemplateChild("splashLoader") as FrameworkElement;
			topLeftLogo = this.GetTemplateChild("topLeftLogo") as FrameworkElement;
			layoutRoot = this.GetTemplateChild("layoutRoot") as FrameworkElement;
			//Top right:
			topRightButtons = this.GetTemplateChild("topRightButtons") as FrameworkElement;
			topRightPanel = this.GetTemplateChild("topRightPanel") as FrameworkElement;
			searchBox = this.GetTemplateChild("searchBox") as FrameworkElement;
			treeViewPanel = this.GetTemplateChild("treeViewPanel") as FrameworkElement;
			examplesPanel = this.GetTemplateChild("examplesPanel") as FrameworkElement;
			examplesMask = this.GetTemplateChild("examplesMask") as FrameworkElement;

			this.Dispatcher.BeginInvoke(() =>
			{
				layoutRoot.Opacity = 1;
				splashLoader.Opacity = 1;
				BeginInitialAnimation();
			});

			//Add cliping:
			(exampleContainer.Parent as Panel).SizeChanged += (sender, e) =>
			{
				var panel = sender as Panel;
				panel.Clip = new System.Windows.Media.RectangleGeometry()
				{
					Rect = new Rect(new Point(), e.NewSize)
				};
			};
#endif
			this.PageLoaded(null, null);
		}

#if !WPF
		private void BeginInitialAnimation()
		{
			var logoWidth = topLeftLogo.ActualWidth;
			//layoutRoot.Opacity = 1;
			var treeWidth = treeViewPanel.Width;
			var exampleWidth = examplesPanel.ActualWidth;
			var buttonsHeight = topRightButtons.ActualHeight;

			var scaleUpSpline = new Spline(0.321000009775162, 0, 0.45100000500679, 1);

			var initialAnimation = splashLoader
						.Animate()
							.Opacity(0, 1, 1, 0)
						.Animate(topLeftLogo)
							.EnsureDefaultTransforms()
							.Opacity(0, 0, 1, 0, 2, 1)
							.MoveX(0, logoWidth, 1, logoWidth, 2, 0)
								.Splines(2, scaleUpSpline)
						.Animate(treeViewPanel, searchBox)
							.EnsureDefaultTransforms()
							.Opacity(0, 0, 2, 0, 3, 1)
							.MoveX(0, -logoWidth, 2, -logoWidth, 3, 0)
								.Splines(2, scaleUpSpline)
						.Animate(examplesMask)
							.Opacity(0, 1, 5, 1, 5.5, 0)
						.Animate(examplesPanel)
							.EnsureDefaultTransforms()
							.Origin(0, 0.5)
							.Scale(0, 0.8, 5, 0.8, 5.5, 1)
								.Splines(2, scaleUpSpline)
							.MoveX(0, exampleWidth, 3, exampleWidth, 5, 0)
								.Splines(2, scaleUpSpline)
						.Animate(topRightButtons, topRightPanel)
							.EnsureDefaultTransforms()
							.MoveY(0, -buttonsHeight, 5.5, -buttonsHeight, 6, 0)
								.Splines(2, scaleUpSpline)
						.Instance;

			initialAnimation.SpeedRatio = 1.5;


			initialAnimation.Completed += OnInitialAnimationCompleted;

			initialAnimation.Begin();
		}

        private void OnInitialAnimationCompleted(object sender, EventArgs args)
        {
            NavigationService service = NavigationService.GetNavigationService();
            
            if (service.RootPanel != null && service.RootPanel.Children.Count > 0)
            {
                BasePage examplePage = service.RootPanel.Children[0] as BasePage;
                if (examplePage != null)
                {
                    examplePage.SelectedExample.AnimationCompleted();
                }
            }
        }
#endif

		public void ShowLoadingVisual()
		{
#if !WPF
			this.LoadingVisual.Begin();
#endif
		}

		public void HideLoadingVisual()
		{
#if !WPF
			this.LoadingVisual.Stop();
#endif
		}

		private void OnSearchTextKeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Enter)
			{
				//this.examplesTree.IsSingleExpandPath = true;
				//this.Examples.ExpandedAll(false);

				SearchForExample();

				//this.Examples.ApplyFilter(string.Empty);
				//searchTextBox.Reset();
			}
		}

		private void SearchForExample()
		{
			ExampleItem searchItem = this.Examples.SelectFirstValidOrDefault(null);

			if (searchItem != null)
			{
				if (!searchItem.IsExpanded)
				{
					searchItem.IsExpanded = true;
				}
				NavigateToPage(searchItem);
			}
		}

		internal void Navigate(ExampleItem item)
		{
			if (item.IsExample)
			{
				string searchType = GetShortExampleTypeName(item.Path);

				Example example = ApplicationCache.LoadTypeFromCache(searchType) as Example;

				if (example != null)
				{
					this.NavigateToExample(example, item);
				}
				else
				{
#if !WPF
					this.LoadingVisual.Begin();
#endif
					ApplicationHelper.LoadDynamicAssembly(item.Path, searchType, result =>
					{
						this.NavigateToExample(result as Example, item);
#if !WPF
						this.LoadingVisual.Stop();
#endif
					});
				}
				this.CurrentSelectedExample = item;
				if (!item.IsSelected)
				{
					item.IsSelected = true;
				}
				this.ConstructBrowserTitle(item);
			};
		}

        private void NavigateToExample(Example example, ExampleItem item)
        {
            var page = BasePage.GetValidPage(item);

            page.SelectedExample = example;
            page.ExampleItem = item;

            this.exampleContainer.NavigateToExample(page);

            //// Using DataContext to bind VersionPresenter Visibility proprty
            this.DataContext = item.Path;
        }

		internal void LoadExamples()
		{
			string uri = BrowserHelper.PathToExampleXML;

			Uri examplesXml = new Uri(uri, UriKind.Relative);

			StreamResourceInfo resource = Application.GetResourceStream(examplesXml);
			if (resource == null)
			{
				throw new MissingManifestResourceException("Cannot find Examples.xml");
			}
			XDocument document = XDocument.Load(new StreamReader(resource.Stream));

			this.Examples = ExampleItem.SelectAll(document.Element("examples"));

			this.examplesTree.ItemsSource = this.Examples;

			if (AsyncDownloadExamples)
			{
				ApplicationHelper.BeginDownloadingXAPs(this.Examples);
			}

			string exampleName = "home";
#if WPF
			string[] commandLineArguments = Application.Current.Properties["args"] as string[];
			if ((commandLineArguments != null) && (commandLineArguments.Length > 0))
			{
				string defaultExampleArgument = ((string[])commandLineArguments)[0];
				if (!string.IsNullOrEmpty(defaultExampleArgument))
				{
					exampleName = defaultExampleArgument;
				}
			}
#endif

			ExampleItem example = this.FindExampleByPath(BrowserHelper.GetCurrentBookmark(exampleName));

			if (example != null)
			{
				Navigate(example);
			}
		}

		private void OnSearchTextChanged(object sender, EventArgs e)
		{
			this.examplesTree.IsSingleExpandPath = string.IsNullOrEmpty(searchTextBox.Text);
			this.Examples.ExpandedAll(!this.examplesTree.IsSingleExpandPath);
			this.Examples.ApplyFilter(searchTextBox.Text);
		}

		private static string GetShortExampleTypeName(string itemPath)
		{
			StringBuilder typeName = new StringBuilder();

			typeName.Append(ApplicationHelper.ExampleApplicationRootNamespace);
#if WPF
			typeName.Append(".");
#else

			if (Regex.IsMatch(itemPath, "^GridView"))
				typeName.Append(".");
			else
				typeName.Append(".Examples.");
#endif

			typeName.Append(itemPath.Replace('/', '.'));
			typeName.Append(".Example");

			return typeName.ToString();
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
		private void NavigateToPage(ExampleItem item)
		{
			if (CheckForValidExample(item))
			{
				return;
			}

			ExampleItem currentItem = item;

			// Check if is main category example
			if (item.Parent == null)
			{
				currentItem = item.CategoryFirstLookExample;
			}

#if !WPF
			BrowserHistoryManager.AddBrowserHistory(currentItem.Path, string.Empty);
#else
			Navigate(currentItem);
#endif
		}

		private void OnBrowserHistoryNavigateCompleted(string path)
		{
			ExampleItem example = this.FindExampleByPath(path);

			if (example != null)
			{
				this.Navigate(example);
			}
		}

		private void OnExamplesTreeSelectionChanged(object sender, RadRoutedEventArgs e)
		{
			if (this.examplesTree.SelectedItem != null)
			{
				ExampleItem selectedExample = this.examplesTree.SelectedItem as ExampleItem;

				if (!selectedExample.IsExpanded)
				{
					selectedExample.IsExpanded = true;
				}

				this.NavigateToPage(selectedExample);
			}
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
		private bool CheckForValidExample(ExampleItem selectedExample)
		{
			if (this.CurrentSelectedExample == null) return false;

			if (!selectedExample.IsExample && selectedExample.Parent != null) return true;

			ExampleItem exampleItem = selectedExample.Children.SelectFirstValidOrDefault(selectedExample);

			//if (!selectedExample.IsExample)
			//{
			//    selectedExample.Children.GetFirstValidExample(ref exampleItem);
			//}
			//else
			//{
			//    exampleItem = selectedExample;
			//}

			return string.Equals(this.CurrentSelectedExample.Path,
								 exampleItem.Path, StringComparison.OrdinalIgnoreCase);
		}

		private void PageLoaded(object sender, RoutedEventArgs e)
		{
			this.LoadExamples();
		}

		private ExampleItem FindExampleByPath(string path)
		{
			return this.FindExampleByPath(this.Examples, path);
		}

		private ExampleItem FindExampleByPath(IEnumerable<ExampleItem> examples, string path)
		{
			foreach (ExampleItem item in examples)
			{
				if (string.Equals(item.Path, path, StringComparison.OrdinalIgnoreCase))
				{
					return item;
				}
				else
				{
					ExampleItem child = this.FindExampleByPath(item.Children, path);
					if (child != null)
					{
						return child;
					}
				}
			}
			return null;
		}

		private void ConstructBrowserTitle(ExampleItem item)
		{
			string titleDescription = string.Empty;

			if (item.IsHomepage)
			{
				titleDescription = string.Format(CultureInfo.InvariantCulture, "Highlights of RadControls {0}", item.Platform);
			}
			else
			{
				string exampleTitle = string.Empty;
				titleDescription = this.GetTitleText(item, ref exampleTitle);
				titleDescription = string.Format(CultureInfo.InvariantCulture, "Telerik Rad{0} {1} {2}", titleDescription, item.Platform, exampleTitle);
			}
			BrowserHelper.SetBrowserTitle(titleDescription);
		}

		private string GetTitleText(ExampleItem item, ref string exampleTitle)
		{
			if (item.Parent != null)
			{
				exampleTitle = string.Format(" - {0}{1}", item.Text, exampleTitle);
				return this.GetTitleText(item.Parent, ref exampleTitle);
			}
			else
			{
				return item.Text;
			}
		}

#if WPF
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings")]
		public static string GetNavigateUrl(DependencyObject obj)
		{
			return (string)obj.GetValue(NavigateUrlProperty);
		}

		public static void SetNavigateUrl(DependencyObject obj, string value)
		{
			obj.SetValue(NavigateUrlProperty, value);
		}

		public static readonly DependencyProperty NavigateUrlProperty =
			DependencyProperty.RegisterAttached("NavigateUrl", typeof(string), typeof(MainPage),
			new PropertyMetadata((DependencyObject d, DependencyPropertyChangedEventArgs args) =>
				{
					Button button = d as Button;
					if (button != null)
					{
						button.Click += new RoutedEventHandler(delegate
							{
								MainPage page = button.ParentOfType<MainPage>();
								if (page != null)
								{
									ExampleItem example = page.FindExampleByPath((string)args.NewValue);
									if (example != null)
									{
										page.Navigate(example);
									}
								}
							});
					}
				}));
#endif
	}
}
